@@ -19,24 +19,29 @@ _counter = count(1)
 
 
 class NoOutput(Exception):
-    def __init__(self, *args: object) -> None:
+    ...
+
+
+class StoppedUnit(Exception):
+    def __init__(self, uid, *args: object) -> None:
         super().__init__(*args)
+        self.uid = uid
 
 
 class PrefetchUnit(Process):
-    def __init__(self, num_prefetch: int, gid: int, ignore_error: bool, processing_func: Optional[Callable[[Any], Any]] = None):
-        """Prefetch Generator
+    def __init__(self, num_prefetch: int, uid: int, ignore_error: bool, processing_func: Optional[Callable[[Any], Any]] = None):
+        """Prefetch Unit
         
         Args:
             num_prefetch (int): prefetch 개수
-            gid (int): generator id
+            uid (int): generator id
             processing_func (Callable): (Optional) 데이터 처리 함수
         """
-        Process.__init__(self, name=f"PrefetchUnit-{gid}")
+        Process.__init__(self, name=f"PrefetchUnit-{uid}")
         self.data = SimpleQueue()
         self.queue = Queue(maxsize=num_prefetch)
         self.num_prefetch = num_prefetch
-        self.gid = gid
+        self.uid = uid
         self.ignore_error = ignore_error
         self.processing_func = processing_func
         self.daemon = True
@@ -50,7 +55,7 @@ class PrefetchUnit(Process):
         while True:
             item = self.data.get()
             if item is StopIteration:
-                self.queue.put((StopIteration, self.gid, idx, True))
+                self.queue.put((StopIteration, self.uid, idx, True))
                 self.data.close()
                 self.queue.close()
                 return
@@ -59,17 +64,17 @@ class PrefetchUnit(Process):
                     item = item if self.processing_func is None else self.processing_func(item)
                     if isinstance(item, Generator):
                         for _item in item:
-                            self.queue.put((_item, self.gid, idx, False))
+                            self.queue.put((_item, self.uid, idx, False))
                             idx += 1
                     else:
-                        self.queue.put((item, self.gid, idx, False))
+                        self.queue.put((item, self.uid, idx, False))
                         idx += 1
                 except Exception as e:
                     if self.ignore_error:
-                        self.queue.put((tb.format_exc(), self.gid, idx, True))
+                        self.queue.put((tb.format_exc(), self.uid, idx, True))
                         idx += 1
                     else:
-                        self.queue.put((tb.format_exc(), self.gid, idx, True))
+                        self.queue.put((tb.format_exc(), self.uid, idx, True))
                         self.data.close()
                         self.queue.close()
                         return
@@ -88,7 +93,7 @@ class PrefetchGenerator(Thread):
         func_args: processing_func의 인자
         func_kwargs: processing_func의 키워드 인자
     """
-    def __init__(self, iterable: Iterable, num_prefetch: int = 1, num_workers: int = 1, ordered: bool = True, shuffle: bool = False, name: Optional[str] = None, ignore_error: bool = True, processing_func: Optional[Callable[[Any], Any]] = None, func_kwargs: Optional[Dict[str, Any]] = {}) -> None:
+    def __init__(self, iterable: Iterable, num_prefetch: int = 1, num_workers: int = 1, ordered: bool = True, shuffle: bool = False, name: Optional[str] = None, ignore_error: bool = True, processing_func: Optional[Callable[[Any], Any]] = None, func_kwargs: Optional[Dict[str, Any]] = {}, start: bool = True) -> None:
         if name is None:
             name = f"PrefetchGenerator-{_counter.__next__():d}"
         Thread.__init__(self, name=name)
@@ -106,17 +111,15 @@ class PrefetchGenerator(Thread):
         self._continue = True
         self._is_running = True
         self._units = self.make_units()
+        self._stopped_units_set = set()  # Thread 내부가 아니라 외부에서 next를 호출할 때 사용하는 인자임
         self.daemon = True
-        self.start()
+        if start:
+            self.start()
 
     def __enter__(self):
         return self
 
     def __exit__(self, type, value, traceback):
-        # TODO: 필요하지 않은 것 같아서 주석처리
-        # 이상 없으면 다음 버전에서 삭제할 것
-        # while self._continue and self.results:
-        #     time.sleep(0.01)
         self.stop()
 
     def _reset_states(self):
@@ -127,12 +130,12 @@ class PrefetchGenerator(Thread):
 
         # self._pendding=True이면 PrefetchUnit에서 데이터를 꺼내야하는 상태를 의미함
 
-        # self._stoped_units는 데이터 처리가 끝난 PrefetchUnit의 개수를 의미하며,
-        # self._stoped_units와 self.num_workers가 같아지면 모든 unit의 작업이 끝난 것을 의미함
+        # self._stopped_units는 데이터 처리가 끝난 PrefetchUnit의 개수를 의미하며,
+        # self._stopped_units와 self.num_workers가 같아지면 모든 unit의 작업이 끝난 것을 의미함
         self.results = OrderedDict()
         self._next_idx = 0
         self._pendding = False
-        self._stoped_units = 0
+        self._stopped_units = 0
 
     def make_units(self) -> Dict[int, PrefetchUnit]:
         """self.num_workers만큼 PrefetchUnit 생성
@@ -142,7 +145,7 @@ class PrefetchGenerator(Thread):
         unit1에는 [1, 3], unit2에는 [2, 4]가 할당됨
 
         Returns:
-            (Dict[int, PrefetchUnit]): gid(PrefetchUnit의 번호)를 key로 가지는 dictionary
+            (Dict[int, PrefetchUnit]): uid(PrefetchUnit의 번호)를 key로 가지는 dictionary
         """
 
         def assign(iterable, units):
@@ -169,7 +172,7 @@ class PrefetchGenerator(Thread):
                 unit.put(StopIteration)
 
         num_prefetch_each_workers = max(self.num_prefetch//self.num_workers + (1 if self.num_prefetch%self.num_workers != 0 else 0) -1, 1)
-        units = {gid: PrefetchUnit(num_prefetch_each_workers, gid, self._ignore_error, self.processing_func) for gid in range(self.num_workers)}
+        units = {uid: PrefetchUnit(num_prefetch_each_workers, uid, self._ignore_error, self.processing_func) for uid in range(self.num_workers)}
         _assign = Thread(target=assign, args=(self.iterable, units), name="Assign")
         _assign.daemon = True
         _assign.start()
@@ -201,24 +204,25 @@ class PrefetchGenerator(Thread):
 
             self._reset_states()
             _c = cycle(range(self.num_workers))  # 데이터를 꺼낼 PrefetchUnit의 index를 순차적으로 출력
-            while self._stoped_units < self.num_workers and self._is_running:
+            while self._stopped_units < self.num_workers and self._is_running:
                 try:
                     if self._pendding:
                         # item: 데이터
-                        # gid: unit의 번호
+                        # uid: unit의 번호
                         # idx: unit 내부의 몇 번째 데이터인지
                         # err: 오류 여부. 오류가 아니면 False
-                        item, gid, idx, err = self._units[next(_c)].queue.get_nowait()
 
-                        # 전체 데이터에서의 index 계산
-                        total_idx = gid+idx*self.num_workers
+                        item, uid, idx, err = self._units[next(_c)].queue.get_nowait()
+                        total_idx = uid+idx*self.num_workers  # 전체 데이터에서의 index 계산
+                        
                         if not err:
                             self.results[total_idx] = item
                         else:
                             if item is StopIteration:
                                 # unit 하나가 종료됨
-                                self._stoped_units += 1
-                                self._units[gid].terminate()
+                                self.results[total_idx] = StoppedUnit(uid)
+                                self._stopped_units += 1
+                                self._units[uid].terminate()
                             else:
                                 if self._ignore_error:
                                     self.results[total_idx] = NoOutput
@@ -244,8 +248,8 @@ class PrefetchGenerator(Thread):
             _run()
 
     def stop_units(self):
-        for gid, unit in self._units.items():
-            self._stoped_units = min(self._stoped_units+1, len(self._units))
+        for uid, unit in self._units.items():
+            self._stopped_units = min(self._stopped_units+1, len(self._units))
             unit.terminate()
 
     def stop(self):
@@ -254,32 +258,44 @@ class PrefetchGenerator(Thread):
         self._continue = False
         self.stop_units()
 
+    def stop_next(self):
+        self._stopped_units_set = set()  # 종료하기 전에 초기화
+        raise StopIteration
+    
     def __next__(self):
         while True:
             if not self._is_running:
                 # run이 완료되지 않은 상태에서 self.stop()이 호출된 경우
                 # 작업중인 unit을 모두 종료하고 StopIteration을 일으킨다.
                 [_unit.terminate() for _unit in self._units.values()]
-                raise StopIteration
+                self.stop_next()
             if not self._continue and not self.results:
                 # iteration이 정상적으로 한 번 끝난 경우
                 # results에서 꺼낼 데이터도 없고, unit도 모두 종료된 상태
-                raise StopIteration
+                self.stop_next()
             if (self._next_idx in self.results if self.ordered else self.results):
                 # self.ordered=True 이면, self.results에서 next_idx를 꺼내고
                 # self.ordered=False 이면, popitem(False)로 가장 오래된 데이터부터 꺼냄
                 item = self.results.pop(self._next_idx) if self.ordered else self.results.popitem(False)[1]
+                self._next_idx += 1
                 if item is NoOutput:
-                    self._next_idx +=1
+                    continue
+                elif isinstance(item, StoppedUnit):
+                    self._stopped_units_set.add(item.uid)
                     continue
                 else:
                     self._pendding = False
-                    self._next_idx +=1
                     return item
             else:
                 # unit에서 데이터가 준비될때까지 대기하기 위해서 진입하는 조건문
                 # sleep이 없는 경우 불필요하게 자원을 많이 사용할 수 있어서 제약을 둠
                 self._pendding = True
+                if self._next_idx % self.num_workers in self._stopped_units_set and self._next_idx not in self.results:
+                    # uid = self._next_idx % self.num_workers
+                    # uid가 종료된 유닛 목록에 있으면 self._next_idx는 절대 self.result에 들어오지 않으므로
+                    # self._next_idx에 1을 더해서 다음 유닛에 대한 결과를 확인하게 함
+                    self._next_idx += 1
+                    continue
                 time.sleep(0.01) 
           
     def __iter__(self):
